/*******************************************
 *******************************************
 *******************************************

Community detection algorithm based on the paper
F.Radicchi et al., Proc. Natl. Acad. Sci. USA 101, 2658-2663 (2004)

Author: Filippo Radicchi
Version: 0.1


 *******************************************
 *******************************************
 *******************************************/





#include <stdio.h>
#include <stdlib.h>





#include "treelib.cpp"









int countnodes (char *filename);
int checkbond(int i, int j, int **bond);
void addlink(int i, int j , int **bond);
int readnetwork (char *filename, int **bond);




double clustering3_edge (int i, int j, int **bond);
double clustering4_edge (int i, int j, int **bond);


void initlist (int N, int **bond, int **listlinks, int **bondlist, tabdeg *CC, int G);



void delete_worst(tabdeg *CC, int *vecworst, int *vectcomm, int *belongs, int **listlinks);
void delete_from_bond(int *vecworst, int **listlinks, int **bond, int **bondlist);
int components (int **group, int ref, int *sourcevec, int *sourcepos, int *boxes, int **bond, int *control, int *tmpvec1, int *tmpvec2);

int create_newgroups(int index, int **group, int ref, int *belongs, int **tree, int *sourcevec, int *sourcepos, int *boxes, int **bond, int *control, int *tmpvec1, int *tmpvec2);


void recalculate (int *vecworst, int **listlinks, int **bond, int **bondlist, tabdeg *CC, int G);


void community_definition (int **group, int ref, int N, int **bond, int *control, int *res);





void printgroup(int **group, int ref, int *strong, int *weak);



void clean_tree(int **tree, int N, int *strong, int *weak, int **group, int Nodes);

int findfather (int i, int *backtree);


void  pajek(int **tree, int **group, int N, char *filename, int *vec);




void tree_leaves (int **bond, int N, int *vec, int *leaves);
void print_leaves(int *leaves, int **group, char *filename, int N);


double modularity (int *belongs, int **bond, int **group, int *control, int N, int M);



int main (int argc, char **argv)
{


  int i, j;


  

  if(argc!=2 && argc!=3)
    {
      printf("Run ./radetal.out network.dat , where network.dat is the name of the file containing the network you want to analyze\n");
      exit(0);
    }

 

  int G=3;
  if (argc==3) G=atoi(argv[2]);
  if(G!=3 && G!=4) {
    printf("Set G=3 or G=4 for calculating edge clustering\n");
    exit(0);
  }


  printf("Running community detection algorithm with edge-clustering calculated with G=%d on the network %s\n",G,argv[1]); 


  int N=countnodes(argv[1]);
  printf("Total number of nodes %d\n",N);


  ////////////////////////////////////////
  int **bond, **oldbond, **bondlist;
  bond=(int **)malloc((N+1)*sizeof(int *));
  bond[0]=(int *)malloc((N+1)*sizeof(int));
  oldbond=(int **)malloc((N+1)*sizeof(int *));
  oldbond[0]=(int *)malloc((N+1)*sizeof(int));
  bondlist=(int **)malloc((N+1)*sizeof(int *));
  for(i=1;i<=N;i++) 
    {
      bond[0][i]=0;
      bond[i]=(int *)malloc(1*sizeof(int));
    }
  ////////////////////////////////////////




  int M=readnetwork(argv[1], bond);
  printf("Total number of links %d\n",M);
  for(i=1;i<=N;i++)
    {
      oldbond[0][i]=bond[0][i];
      oldbond[i]=(int *)malloc((oldbond[0][i]+1)*sizeof(int));
      bondlist[i]=(int *)malloc((bond[0][i]+1)*sizeof(int));
      for(j=1;j<=oldbond[0][i];j++) oldbond[i][j]=bond[i][j];
    }


  



  ///////////////////////////////////////
  int **listlinks, *vecworst;
  vecworst=(int *)malloc((M+1)*sizeof(int));
  listlinks=(int **)malloc((M+1)*sizeof(int *));
  for(i=1;i<=M;i++) listlinks[i]=(int *)malloc(2*sizeof(int));
  tabdeg *CC= new tabdeg; 
  ///////////////////////////////////////




  //components variables
  int *sourcevec, *sourcepos, *boxes, *control, *tmpvec1, *tmpvec2;
  sourcevec=(int *)malloc((N+1)*sizeof(int));
  sourcepos=(int *)malloc((N+1)*sizeof(int));
  boxes=(int *)malloc((N+1)*sizeof(int));
  control=(int *)malloc((N+1)*sizeof(int));
  tmpvec1=(int *)malloc((N+1)*sizeof(int));
  tmpvec2=(int *)malloc((N+1)*sizeof(int));
  ///////////////////////////



  int Ncomm=N*(N+1)/2;

  //communities variables
  int index=1, oldindex,  **group, *belongs, **tree, refg, *vectcomm, *res, *strong, *weak;
  belongs=(int *)malloc((N+1)*sizeof(int));
  group=(int **)malloc((Ncomm+1)*sizeof(int *));
  tree=(int **)malloc((Ncomm+1)*sizeof(int *));
  tree[0]=(int *)malloc((Ncomm+1)*sizeof(int));
  group[0]=(int *)malloc((Ncomm+1)*sizeof(int));
  vectcomm=(int *)malloc((Ncomm+1)*sizeof(int));
  strong=(int *)malloc((Ncomm+1)*sizeof(int));
  weak=(int *)malloc((Ncomm+1)*sizeof(int));
  res=(int *)malloc(2*sizeof(int));
  for(i=1;i<=Ncomm;i++)
    {
      group[0][i]=0;
      tree[0][i]=0;
      tree[i]=(int *)malloc(1*sizeof(int));
      group[i]=(int *)malloc(1*sizeof(int));
    } 
  
  group[1]=(int *)malloc((Ncomm+1)*sizeof(int));
  group[0][1]=N;
  for(i=1;i<=N;i++) 
    {
      group[1][i]=i;
      belongs[i]=1;
    }
  //////////////////////////////////////



  refg=1;
  oldindex=index;
  index=create_newgroups(index, group, refg, belongs, tree, sourcevec, sourcepos, boxes, bond, control, tmpvec1, tmpvec2);
  


  double Q, Qmax=-10.0;
  int *bq;
  bq=(int *)malloc((N+1)*sizeof(int));




  initlist(N, bond, listlinks, bondlist, CC, G);
  
  


  int Mref=M;

  while(Mref>0){
    delete_worst(CC, vecworst, vectcomm, belongs, listlinks);

   
    delete_from_bond(vecworst, listlinks, bond, bondlist);
    recalculate(vecworst, listlinks, bond, bondlist, CC, G);


    Mref-=vecworst[0];

    oldindex=index;
    for(i=1;i<=vectcomm[0];i++)
      {
	index=create_newgroups(index, group, vectcomm[i], belongs, tree, sourcevec, sourcepos, boxes, bond, control, tmpvec1, tmpvec2);
	
      }


    if(index!=oldindex)
      {
	Q=modularity (belongs, oldbond, group, control, N, M);
	if(Q>Qmax)
	  {
	    for(i=1;i<=N;i++) bq[i]=belongs[i];
	    Qmax=Q;
	    //printf("max modularity %g\n",Q);
	  }
      }





  }



  for(i=1;i<=index;i++)
    {
      community_definition(group, i ,N, oldbond, control, res);
      strong[i]=res[0];
      weak[i]=res[1];
    }

  char filename[100];
  sprintf(filename,"wpajek.net");
  pajek(tree, group, index, filename, weak);
  sprintf(filename,"spajek.net");
  pajek(tree, group, index, filename, strong);




  clean_tree(tree, index, strong, weak, group, N);



  FILE *a;
  sprintf(filename,"maxmodularity.dat");
  a=fopen(filename,"w");
  for(i=1;i<=N;i++) fprintf(a,"%d %d\n",i,bq[i]);
  fclose(a);

 



  for(i=0;i<=N;i++) free(bond[i]);
  free(bond);
  for(i=1;i<=N;i++) free(bondlist[i]);
  free(bondlist);
  for(i=0;i<=N;i++) free(oldbond[i]);
  free(oldbond);
  for(i=1;i<=M;i++) free(listlinks[i]);
  free(listlinks);
  for(i=0;i<=N;i++) free(group[i]);
  free(group);
  for(i=0;i<=N;i++) free(tree[i]);
  free(tree);
  free(belongs);
  free(vecworst);
  free(sourcevec); 
  free(sourcepos); 
  free(boxes); 
  free(control); 
  free(tmpvec1); 
  free(tmpvec2);
  free(vectcomm);
  free(res);
  free(strong);
  free(weak);
  free(bq);
 

  return 0;
}




int countnodes (char *filename)
{
  int q, i, j, N=0;
  FILE *a;
  a=fopen(filename,"r");
  while(!feof(a))
    {
      q=fscanf(a,"%d %d\n",&i,&j);
      if(q<=0) goto exitfile;
      if(i>N) N=i;
      if(j>N) N=j;
    }
 exitfile:
  fclose(a);
  return N;
}


int readnetwork (char *filename, int **bond)
{
  int q, i, j, k, M=0;
  FILE *a;
  a=fopen(filename,"r");
  while(!feof(a))
    {
      q=fscanf(a,"%d %d\n",&i,&j);
      if(q<=0) goto exitfile;
      k=checkbond(i,j,bond);
      if(k==-1) 
	{
	  addlink(i,j,bond);
	  M++;
	}
      k=checkbond(j,i,bond);
      if(k==-1) 
	{
	  addlink(j,i,bond);
	  M++;
	}
    }
 exitfile:
  fclose(a);
  return M/2;
}


int checkbond(int i, int j, int **bond)
{
  int k;
  if(bond[0][i]==0) return -1;
  for(k=1;k<=bond[0][i];k++) if(bond[i][k]==j) return k;
  return -1;
}

void addlink(int i, int j , int **bond)
{
  bond[0][i]++;
  bond[i]=(int *)realloc(bond[i],(bond[0][i]+1)*sizeof(int));
  bond[i][bond[0][i]]=j;
}




double clustering4_edge (int i, int j, int **bond)
{

  //printf("%d (%d) %d (%d)\n",i,bond[0][i],j,bond[0][j]);

  int k, s, w, n1, n2 , n=1;
  if(bond[0][i]<=1 || bond[0][j]<=1) return 100.0;
  for(k=1;k<=bond[0][i];k++)
    {
      n1=bond[i][k];
     
      for(s=1;s<=bond[0][j];s++)
	{
	  n2=bond[j][s];
	  for(w=1;w<=bond[0][n2];w++)
	    {
	      if(bond[n2][w]==n1) 
		{
		  n++;
		  //printf("%d --> %d\n",bond[i][k],n);
		}
	    }
	}
	
    }
  double min=(double)(bond[0][i]-1)*(double)(bond[0][j]-1);
  //printf("%d %d %g\n",i,j,(double)n/min);
  return (double)n/min;
}



double clustering3_edge (int i, int j, int **bond)
{

  //printf("%d (%d) %d (%d)\n",i,bond[0][i],j,bond[0][j]);

  int k, s, n=1;
  if(bond[0][i]<=1 || bond[0][j]<=1) return 100.0;
  for(k=1;k<=bond[0][i];k++)
    {
      for(s=1;s<=bond[0][j];s++)
	{
	  if(bond[i][k]==bond[j][s]) 
	    {
	      n++;
	      //printf("%d --> %d\n",bond[i][k],n);
	    }
	}
    }
  int min=bond[0][i]-1;
  if(bond[0][j]<bond[0][i]) min=bond[0][j]-1;
  return (double)n/(double)(min);
}




void initlist (int N, int **bond, int **listlinks, int **bondlist, tabdeg *CC, int G)
{
  double tmp;
  int i, j, k;
  int n1, n2, m=0;
  for(i=1;i<=N;i++)
    {
      for(j=1;j<=bond[0][i];j++)
	{
	  n1=i;
	  n2=bond[i][j];
	  if(n1<n2)
	    {
	      m++;
	      listlinks[m][0]=n1;
	      listlinks[m][1]=n2;
	      if(G==3) tmp=clustering3_edge (n1, n2, bond);
	      if(G==4) tmp=clustering4_edge (n1, n2, bond);
	      CC[0].edinsert(m, tmp); 
	    }
	}
    }
  for(i=1;i<=m;i++)
    {
      n1=listlinks[i][0];
      n2=listlinks[i][1];
      k=checkbond(n1,n2,bond);
      bondlist[n1][k]=i;
      k=checkbond(n2,n1,bond);
      bondlist[n2][k]=i;
    }
}


void delete_worst(tabdeg *CC, int *vecworst, int *vectcomm, int *belongs, int **listlinks)
{
  tabdeg D;
  vecworst[0]=0;
  int g, n, w2, w1=CC[0].worst_node();
  double tw2, tw1=CC[0].indegof(w1);
  CC[0].erase(w1);
  vecworst[0]++;
  vecworst[vecworst[0]]=w1;
  tw2=tw1;

  n=listlinks[w1][0];
  g=belongs[n];
  D.edinsert(g,1.0);
  //printf("Insert group %d\n",g);

  while(tw2==tw1)
    {
      w2=CC[0].worst_node();
      tw2=CC[0].indegof(w2);
      if(tw2==tw1)
	{
	  CC[0].erase(w2);
	  vecworst[0]++;
	  vecworst[vecworst[0]]=w2;
	  
	  n=listlinks[w2][0];
	  g=belongs[n];
	  D.edinsert(g,1.0);
	  //printf("Insert group %d\n",g);

	}
      else{goto next;}
    }
 next:
  vectcomm[0]=0;
  for(muspi::iterator it=D.number_label.begin();it!=D.number_label.end();it++)
    {
      g=it->second;
      vectcomm[0]++;
      vectcomm[vectcomm[0]]=g;
    }
}

void delete_from_bond(int *vecworst, int **listlinks, int **bond, int **bondlist)
{
  int i, j, n1, n2, k1, k2;
  for(i=1;i<=vecworst[0];i++)
    {
      j=vecworst[i];
      n1=listlinks[j][0];
      n2=listlinks[j][1];
      k1=checkbond(n1, n2, bond);
      

      //printf("delete link %d %d %d\n",j,n1,n2);

      if(k1==-1)
	{
	  printf("Error 1: %d  %d %d\n",j,n1,n2);
	  for(j=1;j<=bond[0][n1];j++) printf("%d ",bond[n1][j]);
	  printf("\n");
	  for(j=1;j<=bond[0][n1];j++) printf("%d ",bondlist[n1][j]);
	  printf("\n");
	  exit(0);
	}

      k2=checkbond(n2, n1, bond);
      if(k2==-1)
	{
	  printf("Error 2: %d  %d %d\n",j,n1,n2);
	  for(j=1;j<=bond[0][n2];j++) printf("%d ",bond[n2][j]);
	  printf("\n");
	  for(j=1;j<=bond[0][n2];j++) printf("%d ",bondlist[n2][j]);
	  printf("\n");
	  exit(0);
	}


      bond[n1][k1]=bond[n1][bond[0][n1]];
      bondlist[n1][k1]=bondlist[n1][bond[0][n1]];
      bond[0][n1]--;

      bond[n2][k2]=bond[n2][bond[0][n2]];
      bondlist[n2][k2]=bondlist[n2][bond[0][n2]];
      bond[0][n2]--;

     
    }
}



void recalculate (int *vecworst, int **listlinks, int **bond, int **bondlist, tabdeg *CC, int G)
{
  double clust;
  int j, i, n1, n2, v, k, s, r;
  for(i=1;i<=vecworst[0];i++)
    {
      j=vecworst[i];
      n1=listlinks[j][0];
      n2=listlinks[j][1];

      

      ///////////////////////////////////////
      for(k=1;k<=bond[0][n1];k++)
	{
	  r=bond[n1][k];
	  v=bondlist[n1][k];
	  if(G==3) clust=clustering3_edge(n1, r, bond);
	  if(G==4) clust=clustering4_edge(n1, r, bond);
	  CC[0].erase(v);
	  CC[0].edinsert(v,clust);
	  //printf("aa r %d  n1 %d v %d\n",r,n1,v);
	   
	
	  ///////////////////////////
	  if(G==4)
	    {
	      for(s=1;s<=bond[0][r];s++)
		{
		
		  v=bondlist[r][s];
		  clust=clustering4_edge(r, bond[r][s], bond);
		  CC[0].erase(v);
		  CC[0].edinsert(v,clust);
		}
	    }
	  //////////////////////////////////////////
	}
      ///////////////////////////////////////////////////

      for(k=1;k<=bond[0][n2];k++)
	{
	  r=bond[n2][k];
	  v=bondlist[n2][k];
	  if(G==3) clust=clustering3_edge(n2, r, bond);
	  if(G==4) clust=clustering4_edge(n2, r, bond);
	  CC[0].erase(v);
	  CC[0].edinsert(v,clust);
	  //printf("aa r %d  n2 %d v %d\n",r,n2,v);
	  
	  /////////////////////////////////////
	  if(G==4)
	    {
	      for(s=1;s<=bond[0][r];s++)
		{

		  v=bondlist[r][s];
		  clust=clustering4_edge(r, bond[r][s], bond);
		  CC[0].erase(v);
		  CC[0].edinsert(v,clust);
		  
		}
	    }
	  /////////////////////////////////////////
	}
      //////////////////////////////////////////////

    }
}




void printgroup(int **group, int ref, int *strong, int *weak)
{
  int i;
 

  if(strong[ref]==1 || weak[ref]==1){
    printf("Group %d\n",ref);
    printf("Number of nodes %d\n",group[0][ref]);
    printf("Nodes belonging to the group: ");
    for(i=1;i<=group[0][ref];i++) printf("%d ",group[ref][i]);
    printf("\n");
  
    printf("Strong community: ");
    if(strong[ref]==1) printf("yes\n");
    else{printf("no\n");}
    printf("Weak community: ");
    if(weak[ref]==1) printf("yes\n");
    else{printf("no\n");}
    printf("\n\n------------\n------------\n------------\n\n");
  }
}










void print_leaves(int *leaves, int **group, char *filename, int N)
{
  int *vv, i, j, n, r;
  vv=(int *)malloc((N+1)*sizeof(int));
  for(i=1;i<=N;i++) vv[i]=-1;

  for(i=1;i<=leaves[0];i++)
    {
      n=leaves[i];
      for(j=1;j<=group[0][n];j++)
	{
	  r=group[n][j];
	  if(vv[r]>0) printf("Error: %d %d\n",r,vv[r]);
	  vv[r]=i;
	}
    }

  FILE *a;
  a=fopen(filename,"w");
  for(i=1;i<=N;i++) fprintf(a,"%d %d\n",i,vv[i]);
  fclose(a);


  free(vv);
}












int components (int **group, int ref, int *sourcevec, int *sourcepos, int *boxes, int **bond, int *control, int *tmpvec1, int *tmpvec2)
{

  int source, node,node2,i,j,d,lnode;



  sourcevec[0]=group[0][ref];
  boxes[0]=0;
  for(i=1;i<=group[0][ref];i++)
    {
      sourcevec[i]=group[ref][i];
      sourcepos[group[ref][i]]=i;
      boxes[group[ref][i]]=-1;
      control[group[ref][i]]=-1;
    }

  

  while(sourcevec[0]>0)
    {

      
      
      source=(int)(rand()/((double)RAND_MAX + 1)*(float)(sourcevec[0]))+1;
      if(source>sourcevec[0]) source=1;
      if(source<1) source=sourcevec[0];
      source=sourcevec[source];
      
      lnode=sourcevec[sourcevec[0]];
      sourcevec[sourcepos[source]]=lnode;
      sourcepos[lnode]=sourcepos[source];
      sourcepos[source]=-1;
      sourcevec[0]--;

      

      

      d=0;
      control[source]=d;
      

      
      tmpvec1[0]=1;
      tmpvec1[tmpvec1[0]]=source;

      
      
      boxes[0]++;
      boxes[source]=boxes[0];
      
      

      while(tmpvec1[0]>0)
	{
          d++;

	  
          tmpvec2[0]=0;
          for(i=1;i<=tmpvec1[0];i++)
            {
	      node=tmpvec1[i];
              
	      for(j=1;j<=bond[0][node];j++)
		{
                  node2=bond[node][j];
		  
                  if(control[node2]==-1)
                    {
                      control[node2]=d;
		      tmpvec2[0]++;
                      tmpvec2[tmpvec2[0]]=node2;
	            }
                  if(boxes[node2]==-1)
                    {
                      boxes[node2]=boxes[0];
                      if(sourcepos[node2]>0)
                        {
                          lnode=sourcevec[sourcevec[0]];
			  sourcevec[sourcepos[node2]]=lnode;
                          sourcepos[lnode]=sourcepos[node2];
			  sourcepos[node2]=-1;
		          sourcevec[0]--;
                        }
		      
		      
                    }
		  
		}
            }
          for(i=0;i<=tmpvec2[0];i++) tmpvec1[i]=tmpvec2[i];
        }
      
      
    }
  
  
  
  return boxes[0];

}


int create_newgroups(int index, int **group, int ref, int *belongs, int **tree, int *sourcevec, int *sourcepos, int *boxes, int **bond, int *control, int *tmpvec1, int *tmpvec2)
{
  int comp=components(group, ref, sourcevec, sourcepos, boxes, bond, control, tmpvec1, tmpvec2);
  //printf("comp number %d\n",comp);
  if(comp==1) return index;
  int i,n,g;
  for(i=1;i<=comp;i++) group[i+index][0]=0;
  for(i=1;i<=group[0][ref];i++)
    {
      n=group[ref][i];
      g=boxes[n]+index;
      addlink(g,n,group);
      belongs[n]=g;
      //printf("n %d ---> g %d\n",n,g);
    }
  
 for(i=1;i<=comp;i++) addlink(ref,i+index,tree);

 return index+comp;
}






void community_definition (int **group, int ref, int N, int **bond, int *control, int *res)
{
  int kin, kout, min=0, mout=0, i, j, n, c1=1, c2=1;
  for(i=1;i<=N;i++) control[i]=-1;
  for(i=1;i<=group[0][ref];i++) control[group[ref][i]]=1;
  for(i=1;i<=group[0][ref];i++)
    {
      kin=0;
      kout=0;
      n=group[ref][i];
      for(j=1;j<=bond[0][n];j++) 
	{
	  if(control[bond[n][j]]==1) kin++;
	  else{kout++;}
	}
      if(kin<kout) c1=0;
      min+=kin;
      mout+=kout;
    }
  if(min<mout) c2=0;
  //printf("ref %d  min %d  mout %d\n",ref,min,mout);
  res[0]=c1;
  res[1]=c2;
}







void clean_tree(int **tree, int N, int *strong, int *weak, int **group, int Nodes)
{

  int i, j, n, cs, cw;
  int *backtree, **stree, **wtree;
  backtree=(int *)malloc((N+1)*sizeof(int));

 
  stree=(int **)malloc((N+1)*sizeof(int *));
  wtree=(int **)malloc((N+1)*sizeof(int *));
  wtree[0]=(int *)malloc((N+1)*sizeof(int));
  stree[0]=(int *)malloc((N+1)*sizeof(int));
  for(i=1;i<=N;i++)
    {
      wtree[0][i]=0;
      stree[0][i]=0;
      stree[i]=(int *)malloc(1*sizeof(int));
      wtree[i]=(int *)malloc(1*sizeof(int));
    }
  

  for(i=1;i<=N;i++) backtree[i]=-1;
  for(i=1;i<=N;i++)
    {
      for(j=1;j<=tree[0][i];j++)
	{
	  n=tree[i][j];
	  backtree[n]=i;
	}
    }

  
  for(i=1;i<=N;i++)
    {
      cs=0;
      cw=0;
      for(j=1;j<=tree[0][i];j++)
	{
	  n=tree[i][j];
	  if(weak[n]==1)  cw++;
	  if(strong[n]==1)  cs++;
	}
      if(cw>=2 && weak[i]==1)
	{
	  for(j=1;j<=tree[0][i];j++)
	    {
	      n=tree[i][j];
	      if(weak[n]==1) addlink(i,n,wtree);
	    }
	}
      else{for(j=1;j<=tree[0][i];j++) weak[tree[i][j]]=0;}

      if(cs>=2 && strong[i]==1)
	{
	  for(j=1;j<=tree[0][i];j++)
	    {
	      n=tree[i][j];
	      if(strong[n]==1) addlink(i,n,stree);
	    }
	}
      else{for(j=1;j<=tree[0][i];j++) strong[tree[i][j]]=0;}
    }



  for(i=1;i<=N;i++)
    {
      if(group[0][i]==1)
	{
	  cw=0;
	  j=i;
	  while(cw==0 && j>0)
	    {
	      j=findfather(j, backtree);
	      cw=weak[j];
	    }
	  if(j<0 || cw==0) printf("Error: %d %d\n",cw,j);
	  else{addlink(j,i,wtree);}
	}
    }



  for(i=1;i<=N;i++)
    {
      if(group[0][i]==1)
	{
	  cs=0;
	  j=i;
	  while(cs==0 && j>0)
	    {
	      j=findfather(j, backtree);
	      cs=strong[j];
	    }
	  if(j<0 || cw==0) printf("Error: %d %d\n",cw,j);
	  else{addlink(j,i,stree);}
	}
    }












  char filename[100];
  sprintf(filename,"weak.net");
  pajek(wtree, group, N, filename,weak);
  sprintf(filename,"strong.net");
  pajek(stree, group, N, filename,strong);




  int *leaves;
  leaves=(int *)malloc((N+1)*sizeof(int));

  //printf("Weak\n");
  tree_leaves (wtree, N, weak, leaves);
  sprintf(filename,"weak.dat");
  print_leaves(leaves, group, filename, Nodes);
  //printf("\n\n");


  //printf("Strong\n");
  tree_leaves (stree, N, strong, leaves);
  sprintf(filename,"strong.dat");
  print_leaves(leaves, group, filename, Nodes);
  //printf("\n\n");





  for(i=0;i<=N;i++)
    {
      free(stree[i]);
      free(wtree[i]);
    }
  free(wtree);
  free(stree);
  free(backtree);
  free(leaves);
}





void  pajek(int **tree, int **group, int N, char *filename, int *vec)
{
  int i, j, *index;
  index=(int *)malloc((N+1)*sizeof(int));
  for(i=1;i<=N;i++) index[i]=-1;
  index[0]=0;
  for(i=1;i<=N;i++)
    {
      if(tree[0][i]>0 || group[0][i]==1)
	{
	  index[0]++;
	  index[i]=index[0];
	}
    }


  FILE *a;
  a=fopen(filename,"w");
  fprintf(a,"*Vertices %d\n",index[0]);
  fprintf(a,"%d \"\" triangle x_fact 3 y_fact 3 ic Green\n",index[1]);
  for(i=2;i<=N;i++) 
    {
      
      if(group[0][i]>1 && index[i]>0 && vec[i]==1) fprintf(a,"%d \"\" box x_fact 2 y_fact 2 ic Blue\n",index[i]);
      if(group[0][i]>1 && index[i]>0 && vec[i]==0) fprintf(a,"%d \"\" box x_fact 2 y_fact 2 ic Yellow\n",index[i]);
      if(group[0][i]==1) fprintf(a,"%d \"%d\" ellipse x_fact 1 y_fact 1 ic Red\n",index[i],group[i][1]);
    }
  fprintf(a,"*Arcs\n");
  for(i=1;i<=N;i++) for(j=1;j<=tree[0][i];j++) fprintf(a,"%d %d 1\n",index[i],index[tree[i][j]]);
  fclose(a);
  free(index);


  char prompt[100];
  sprintf(prompt,"unix2dos %s",filename);
  int q=system(prompt);

}



int findfather (int i, int *backtree)
{
  return backtree[i];
}











void tree_leaves (int **bond, int N, int *vec, int *leaves)
{

  int source, node,node2,i,j,d, c;


 
  int *tmpvec1, *tmpvec2;
  
 
  tmpvec1=(int *)malloc((N+1)*sizeof(int));
  tmpvec2=(int *)malloc((N+1)*sizeof(int));

  leaves[0]=0;
  
  
 
      
      
  source=1;
  
  d=0;
 
      
  
  
  tmpvec1[0]=1;
  tmpvec1[tmpvec1[0]]=source;

      
      
    
      

  while(tmpvec1[0]>0)
    {
     	  
      tmpvec2[0]=0;
      for(i=1;i<=tmpvec1[0];i++)
	{
	  node=tmpvec1[i];
	  c=0;
	  for(j=1;j<=bond[0][node];j++)
	    {
	      node2=bond[node][j];
	      if(vec[node2]==1) c++;
	    }
	  //printf("%d %d %d\n",node,c,bond[0][node]);
	  if(c==bond[0][node])
	    {
	      for(j=1;j<=bond[0][node];j++)
		{
		  node2=bond[node][j];
		  tmpvec2[0]++;
		  tmpvec2[tmpvec2[0]]=node2;
		}
	    }
	  else{
	    leaves[0]++;
	    leaves[leaves[0]]=node;
	  }
	      
	    
	}
      for(i=0;i<=tmpvec2[0];i++) tmpvec1[i]=tmpvec2[i];
    }
      
      
    
  
  

free(tmpvec1);
free(tmpvec2);
  
  

}








/////////////////////////////////////////////


double modularity (int *belongs, int **bond, int **group, int *control, int N, int M)
{
  tabdeg D;
  int i, g, j, k, v, n1, n2;
  double m=0.0, mm=0.0;
  for(i=1;i<=N;i++)
    {
      g=belongs[i];
      D.edinsert(g,1.0);
    }
  control[0]=0;
  for(muspi::iterator it=D.number_label.begin();it!=D.number_label.end();it++)
    {
      g=it->second;
      control[0]++;
      control[control[0]]=g;
    }
  for(i=1;i<=control[0];i++)
    {
      g=control[i];
      for(j=1;j<group[0][g];j++) 
	{
	  n1=group[g][j];
	  for(k=i+1;k<=group[0][g];k++) 
	    {
	      n2=group[g][k];
	      v=checkbond(n1,n2,bond);
	      if(v>0) m+=1.0;
	      mm+=(double)bond[0][n1]*(double)bond[0][n2]/2.0/(double)M;
	    }
	}
    }
  //printf("%g %g\n",m,mm);
  double q=(m-mm)/2.0/(double)M;
  return q;
}

